/****************************************************************************
** Meta object code from reading C++ file 'kiconloader.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kiconthemes-6.28.0/src/kiconloader.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kiconloader.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11KIconLoaderE_t {};
} // unnamed namespace

template <> constexpr inline auto KIconLoader::qt_create_metaobjectdata<qt_meta_tag_ZN11KIconLoaderE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KIconLoader",
        "iconLoaderSettingsChanged",
        "",
        "iconChanged",
        "group",
        "newIconLoader",
        "emitChange",
        "Group",
        "_k_refreshIcons",
        "Context",
        "Any",
        "Action",
        "Application",
        "Device",
        "MimeType",
        "Animation",
        "Category",
        "Emblem",
        "Emote",
        "International",
        "Place",
        "StatusIcon",
        "Type",
        "Fixed",
        "Scalable",
        "Threshold",
        "MatchType",
        "MatchExact",
        "MatchBest",
        "MatchBestOrGreaterSize",
        "NoGroup",
        "Desktop",
        "FirstGroup",
        "Toolbar",
        "MainToolbar",
        "Small",
        "Panel",
        "Dialog",
        "LastGroup",
        "User",
        "StdSizes",
        "SizeSmall",
        "SizeSmallMedium",
        "SizeMedium",
        "SizeLarge",
        "SizeHuge",
        "SizeEnormous",
        "States",
        "DefaultState",
        "ActiveState",
        "DisabledState",
        "SelectedState",
        "LastState"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'iconLoaderSettingsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'iconChanged'
        QtMocHelpers::SignalData<void(int)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
        // Slot 'newIconLoader'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'emitChange'
        QtMocHelpers::SlotData<void(enum Group)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 4 },
        }}),
        // Slot '_k_refreshIcons'
        QtMocHelpers::SlotData<void(int)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 4 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Context'
        QtMocHelpers::EnumData<enum Context>(9, 9, QMC::EnumFlags{}).add({
            {   10, Context::Any },
            {   11, Context::Action },
            {   12, Context::Application },
            {   13, Context::Device },
            {   14, Context::MimeType },
            {   15, Context::Animation },
            {   16, Context::Category },
            {   17, Context::Emblem },
            {   18, Context::Emote },
            {   19, Context::International },
            {   20, Context::Place },
            {   21, Context::StatusIcon },
        }),
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(22, 22, QMC::EnumFlags{}).add({
            {   23, Type::Fixed },
            {   24, Type::Scalable },
            {   25, Type::Threshold },
        }),
        // enum 'MatchType'
        QtMocHelpers::EnumData<enum MatchType>(26, 26, QMC::EnumFlags{}).add({
            {   27, MatchType::MatchExact },
            {   28, MatchType::MatchBest },
            {   29, MatchType::MatchBestOrGreaterSize },
        }),
        // enum 'Group'
        QtMocHelpers::EnumData<enum Group>(7, 7, QMC::EnumFlags{}).add({
            {   30, Group::NoGroup },
            {   31, Group::Desktop },
            {   32, Group::FirstGroup },
            {   33, Group::Toolbar },
            {   34, Group::MainToolbar },
            {   35, Group::Small },
            {   36, Group::Panel },
            {   37, Group::Dialog },
            {   38, Group::LastGroup },
            {   39, Group::User },
        }),
        // enum 'StdSizes'
        QtMocHelpers::EnumData<enum StdSizes>(40, 40, QMC::EnumFlags{}).add({
            {   41, StdSizes::SizeSmall },
            {   42, StdSizes::SizeSmallMedium },
            {   43, StdSizes::SizeMedium },
            {   44, StdSizes::SizeLarge },
            {   45, StdSizes::SizeHuge },
            {   46, StdSizes::SizeEnormous },
        }),
        // enum 'States'
        QtMocHelpers::EnumData<enum States>(47, 47, QMC::EnumFlags{}).add({
            {   48, States::DefaultState },
            {   49, States::ActiveState },
            {   50, States::DisabledState },
            {   51, States::SelectedState },
            {   52, States::LastState },
        }),
    };
    return QtMocHelpers::metaObjectData<KIconLoader, qt_meta_tag_ZN11KIconLoaderE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KIconLoader::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KIconLoaderE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KIconLoaderE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KIconLoaderE_t>.metaTypes,
    nullptr
} };

void KIconLoader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KIconLoader *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->iconLoaderSettingsChanged(); break;
        case 1: _t->iconChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->newIconLoader(); break;
        case 3: _t->emitChange((*reinterpret_cast<std::add_pointer_t<enum Group>>(_a[1]))); break;
        case 4: _t->d->_k_refreshIcons((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KIconLoader::*)()>(_a, &KIconLoader::iconLoaderSettingsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KIconLoader::*)(int )>(_a, &KIconLoader::iconChanged, 1))
            return;
    }
}

const QMetaObject *KIconLoader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KIconLoader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KIconLoaderE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int KIconLoader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KIconLoader::iconLoaderSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KIconLoader::iconChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
